@@ -13,7 +13,7 @@ extern "C" {
 
 
 /** Defines the Baresip version string */
-#define BARESIP_VERSION "3.2.0"
+#define BARESIP_VERSION "3.9.0"
 
 
 #ifndef NET_MAX_NS
@@ -72,6 +72,13 @@ enum sipansbeep {
 	SIPANSBEEP_LOCAL,
 };
 
+/** Jitter buffer type */
+enum jbuf_type {
+	JBUF_OFF,
+	JBUF_FIXED,
+	JBUF_ADAPTIVE
+};
+
 struct account;
 
 int account_alloc(struct account **accp, const char *sipaddr);
@@ -138,6 +145,8 @@ int account_answerdelay(const struct account *acc);
 void account_set_answerdelay(struct account *acc, int adelay);
 bool account_sip_autoanswer(const struct account *acc);
 void account_set_sip_autoanswer(struct account *acc, bool allow);
+bool account_sip_autoredirect(const struct account *acc);
+void account_set_sip_autoredirect(struct account *acc, bool allow);
 enum sipansbeep account_sipansbeep(const struct account *acc);
 void account_set_sipansbeep(struct account *acc, enum sipansbeep beep);
 void account_set_autelev_pt(struct account *acc, uint32_t pt);
@@ -212,6 +221,7 @@ bool call_has_audio(const struct call *call);
 bool call_has_video(const struct call *call);
 bool call_early_video_available(const struct call *call);
 bool call_refresh_allowed(const struct call *call);
+bool call_ack_pending(const struct call *call);
 int  call_transfer(struct call *call, const char *uri);
 int  call_replace_transfer(struct call *target_call, struct call *source_call);
 int  call_status(struct re_printf *pf, const struct call *call);
@@ -245,15 +255,19 @@ struct call  *call_find_linenum(const struct list *calls, uint32_t linenum);
 struct call  *call_find_id(const struct list *calls, const char *id);
 void call_set_current(struct list *calls, struct call *call);
 const struct list *call_get_custom_hdrs(const struct call *call);
-int call_set_media_direction(struct call *call, enum sdp_dir a,
+void call_set_media_direction(struct call *call, enum sdp_dir a,
 			     enum sdp_dir v);
-int call_set_media_estdir(struct call *call, enum sdp_dir a, enum sdp_dir v);
+void call_set_media_estdir(struct call *call, enum sdp_dir a, enum sdp_dir v);
 void call_start_answtmr(struct call *call, uint32_t ms);
 bool          call_supported(struct call *call, uint16_t tags);
 const char   *call_user_data(const struct call *call);
 int call_set_user_data(struct call *call, const char *user_data);
 void call_set_evstop(struct call *call, bool stop);
 bool call_is_evstop(struct call *call);
+int call_msg_src(const struct call *call, struct sa *sa);
+enum sip_transp call_transp(const struct call *call);
+enum sdp_neg_state call_sdp_neg_state(const struct call *call);
+bool call_sdp_change_allowed(const struct call *call);
 
 /*
  * Custom headers
@@ -314,6 +328,15 @@ enum audio_mode {
 	AUDIO_MODE_THREAD,           /**< Use dedicated thread          */
 };
 
+/** RTP receive mode */
+enum rtp_receive_mode {
+	RECEIVE_MODE_MAIN = 0,  /**< RTP RX is processed in main thread      */
+	RECEIVE_MODE_THREAD,    /**< RTP RX is processed in separate thread  */
+};
+
+enum rtp_receive_mode resolve_receive_mode(const struct pl *fmt);
+const char *rtp_receive_mode_str(enum rtp_receive_mode rxmode);
+
 
 /** SIP User-Agent */
 struct config_sip {
@@ -393,6 +416,7 @@ struct config_avt {
 	bool rtp_stats;         /**< Enable RTP statistics          */
 	uint32_t rtp_timeout;   /**< RTP Timeout in seconds (0=off) */
 	bool bundle;            /**< Media Multiplexing (BUNDLE)    */
+	enum rtp_receive_mode rxmode;   /**< RTP RX processing mode */
 };
 
 /** Network Configuration */
@@ -808,7 +832,7 @@ enum ua_event {
 	UA_EVENT_CALL_ESTABLISHED,
 	UA_EVENT_CALL_CLOSED,
 	UA_EVENT_CALL_TRANSFER,
-	UA_EVENT_CALL_BLIND_TRANSFER,
+	UA_EVENT_CALL_REDIRECT,
 	UA_EVENT_CALL_TRANSFER_FAILED,
 	UA_EVENT_CALL_DTMF_START,
 	UA_EVENT_CALL_DTMF_END,
@@ -820,8 +844,11 @@ enum ua_event {
 	UA_EVENT_AUDIO_ERROR,
 	UA_EVENT_CALL_LOCAL_SDP,      /**< param: offer or answer */
 	UA_EVENT_CALL_REMOTE_SDP,     /**< param: offer or answer */
+	UA_EVENT_CALL_HOLD,           /**< Call put on-hold by peer          */
+	UA_EVENT_CALL_RESUME,         /**< Call resumed by peer              */
 	UA_EVENT_REFER,
 	UA_EVENT_MODULE,
+	UA_EVENT_END_OF_FILE,
 	UA_EVENT_CUSTOM,
 
 	UA_EVENT_MAX,
@@ -897,6 +924,8 @@ int  ua_call_alloc(struct call **callp, struct ua *ua,
 struct call *ua_find_call_state(const struct ua *ua, enum call_state st);
 int ua_raise(struct ua *ua);
 int ua_set_autoanswer_value(struct ua *ua, const char *value);
+void ua_add_extension(struct ua *ua, const char *extension);
+void ua_remove_extension(struct ua *ua, const char *extension);
 
 
 /* One instance */
@@ -1192,15 +1221,22 @@ struct videnc_state;
 struct viddec_state;
 struct vidcodec;
 
+struct viddec_packet {
+	bool intra;		      /**< True=keyframe, False=deltaframe   */
+	const struct rtp_header *hdr; /**< RTP Header                        */
+	uint64_t timestamp;	      /**< Timestamp in VIDEO_TIMEBASE units */
+	struct mbuf *mb;	      /**< RTP Buffer memory                 */
+};
+
 typedef int (videnc_packet_h)(bool marker, uint64_t rtp_ts,
 			      const uint8_t *hdr, size_t hdr_len,
 			      const uint8_t *pld, size_t pld_len,
-			      void *arg);
+			      const struct video *vid);
 
 typedef int (videnc_update_h)(struct videnc_state **vesp,
 			      const struct vidcodec *vc,
 			      struct videnc_param *prm, const char *fmtp,
-			      videnc_packet_h *pkth, void *arg);
+			      videnc_packet_h *pkth, const struct video *vid);
 
 typedef int (videnc_encode_h)(struct videnc_state *ves, bool update,
 			      const struct vidframe *frame,
@@ -1209,11 +1245,12 @@ typedef int (videnc_encode_h)(struct videnc_state *ves, bool update,
 typedef int (videnc_packetize_h)(struct videnc_state *ves,
 				 const struct vidpacket *packet);
 
-typedef int (viddec_update_h)(struct viddec_state **vdsp,
-			      const struct vidcodec *vc, const char *fmtp);
+typedef int(viddec_update_h)(struct viddec_state **vdsp,
+			     const struct vidcodec *vc, const char *fmtp,
+			     const struct video *vid);
+
 typedef int (viddec_decode_h)(struct viddec_state *vds, struct vidframe *frame,
-                              bool *intra, bool marker, uint16_t seq,
-                              struct mbuf *mb);
+                              struct viddec_packet *pkt);
 
 struct vidcodec {
 	struct le le;
@@ -1377,6 +1414,8 @@ int   video_set_fullscreen(struct video *v, bool fs);
 void  video_vidsrc_set_device(struct video *v, const char *dev);
 int   video_set_source(struct video *v, const char *name, const char *dev);
 void  video_set_devicename(struct video *v, const char *src, const char *disp);
+const char *video_get_src_dev(const struct video *v);
+const char *video_get_disp_dev(const struct video *v);
 int   video_debug(struct re_printf *pf, const struct video *v);
 struct stream *video_strm(const struct video *v);
 const struct vidcodec *video_codec(const struct video *vid, bool tx);
@@ -1426,7 +1465,7 @@ int  stream_start_mediaenc(struct stream *strm);
 int  stream_start_rtcp(const struct stream *strm);
 int  stream_enable(struct stream *strm, bool enable);
 int  stream_enable_tx(struct stream *strm, bool enable);
-int stream_open_natpinhole(struct stream *strm);
+int  stream_enable_rx(struct stream *strm, bool enable);
 void stream_mnat_attr(struct stream *strm, const char *name,
 		      const char *value);
 void stream_set_session_handlers(struct stream *strm,
@@ -1444,6 +1483,38 @@ int  stream_debug(struct re_printf *pf, const struct stream *s);
 void stream_enable_rtp_timeout(struct stream *strm, uint32_t timeout_ms);
 
 
+/**
+ * Jitter Buffer
+ */
+struct jbuf;
+struct rtp_header;
+
+/** Jitter buffer statistics */
+struct jbuf_stat {
+	uint32_t n_put;        /**< Number of frames put into jitter buffer */
+	uint32_t n_get;        /**< Number of frames got from jitter buffer */
+	uint32_t n_oos;        /**< Number of out-of-sequence frames        */
+	uint32_t n_dups;       /**< Number of duplicate frames detected     */
+	uint32_t n_late;       /**< Number of frames arriving too late      */
+	uint32_t n_lost;       /**< Number of lost frames                   */
+	uint32_t n_overflow;   /**< Number of overflows                     */
+	uint32_t n_underflow;  /**< Number of underflows                    */
+	uint32_t n_flush;      /**< Number of times jitter buffer flushed   */
+};
+
+
+int  jbuf_alloc(struct jbuf **jbp, uint32_t min, uint32_t max);
+int  jbuf_set_type(struct jbuf *jb, enum jbuf_type jbtype);
+int  jbuf_put(struct jbuf *jb, const struct rtp_header *hdr, void *mem);
+int  jbuf_get(struct jbuf *jb, struct rtp_header *hdr, void **mem);
+int  jbuf_drain(struct jbuf *jb, struct rtp_header *hdr, void **mem);
+void jbuf_flush(struct jbuf *jb);
+int  jbuf_stats(const struct jbuf *jb, struct jbuf_stat *jstat);
+int  jbuf_debug(struct re_printf *pf, const struct jbuf *jb);
+uint32_t jbuf_frames(const struct jbuf *jb);
+uint32_t jbuf_packets(const struct jbuf *jb);
+
+
 /*
  * STUN URI
  */
@@ -1698,12 +1769,12 @@ int  peerconnection_new(struct peer_connection **pcp,
 		        peerconnection_gather_h *gatherh,
 		        peerconnection_estab_h,
 		        peerconnection_close_h *closeh, void *arg);
-int  peerconnection_add_audio_track(struct peer_connection *pc,
-			 const struct config *cfg,
-			 struct list *aucodecl);
-int  peerconnection_add_video_track(struct peer_connection *pc,
-			 const struct config *cfg,
-			 struct list *vidcodecl);
+int peerconnection_add_audio_track(struct peer_connection *pc,
+				   const struct config *cfg,
+				   struct list *aucodecl, enum sdp_dir dir);
+int peerconnection_add_video_track(struct peer_connection *pc,
+				   const struct config *cfg,
+				   struct list *vidcodecl, enum sdp_dir dir);
 int  peerconnection_set_remote_descr(struct peer_connection *pc,
 				    const struct session_description *sd);
 int  peerconnection_create_offer(struct peer_connection *sess,
